unit BabelGoogle;

interface

uses
  Classes, idHttp, Controls, Forms;

type
  TBabelGoogle = class (TComponent) // non visual...
  protected
    Http1: TIdHttp;
    FFromLang: string;
    FToLang: string;
    FActiveInForm: Boolean;
    procedure SetFromLang(const Value: string);
    procedure SetToLang(const Value: string);
    procedure SetActiveInForm(const Value: Boolean);
  public
    function DoTranslate (strIn: string): string; virtual;
    constructor Create(AOwner: TComponent); override;

    class function Translate (strIn, langIn, langOut: string): string;
  published
    property FromLang: string read FFromLang write SetFromLang;
    property ToLang: string read FToLang write SetToLang;
    property ActiveInForm: Boolean read FActiveInForm write SetActiveInForm;
  end;

  TBabelGoogleRest = class (TBabelGoogle)
  public
    function DoTranslate(strIn: string): string; override;
  end;

  procedure Register;

implementation

uses
  SysUtils, idUri, StrUtils, GoogleTranslateForm;

procedure Register;
begin
  RegisterComponents('FunSide', [TBabelGoogle, TBabelGoogleRest]);
end;


{ TBabelGoogle }

constructor TBabelGoogle.Create(AOwner: TComponent);
begin
  inherited;
  Http1 := TIdHTTP.Create(self);
end;

function TBabelGoogle.DoTranslate(strIn: string): string;
//var
//  strUrl: string;
//  nPosA, nPosB: Integer;
begin
//  strUrl := Format (
//    'http://translate.google.com/translate_t?hl=%s#%s|%s|%s',
//    [FFromLang, FFromLang, FToLang, strIn]);
//  strUrl := TIdUri.URLEncode(strUrl);
//  strResult := Http1.Get('http://translate.google.com/translate_t?hl=en#en|bg|nothing%20here');
//  nPosA := Pos ('id=result_box', strResult); // open tag
//  nPosA := PosEx ('>', strResult, nPosA) + 1; // open tag end
//  nPosB := PosEx ('<', strResult, nPosA) - 1;
//  Result := Copy (strResult, nPosA, nPosB - nPosA);
  Result := GoogleTranslateForm.FormGoogleTranslate.
    DoTranslate(strIn, FFromLang, FToLang);
end;

procedure TBabelGoogle.SetActiveInForm(const Value: Boolean);
var
  fform: TForm;
  I: Integer;
  strCaption: string;
begin
  FActiveInForm := Value;

  if FActiveInForm and not (csDesigning in ComponentState) then
  begin
    if Owner is TForm then
    begin
      FForm := Owner as TForm;
      for I := 0 to FForm.ControlCount - 1 do
      begin
        strCaption := '';
        SetLength (strCaption, 100);
        FForm.Controls[I].GetTextBuf(PChar (strCaption), 100);
        // fix string length
        SetLength (strCaption, StrLen(PChar(strCaption)));
        strCaption := DoTranslate(strCaption);
        FForm.Controls[I].SetTextBuf(PChar(strCaption));
      end;
    end;
  end;
end;

procedure TBabelGoogle.SetFromLang(const Value: string);
begin
  FFromLang := Value;
end;

procedure TBabelGoogle.SetToLang(const Value: string);
begin
  FToLang := Value;
end;

class function TBabelGoogle.Translate(strIn, langIn, langOut: string): string;
var
  bg: TBabelGoogle;
begin
  bg := TBabelGoogle.Create(nil);
  try
    bg.FromLang := langin;
    bg.ToLang := langout;
    Result := bg.DoTranslate(strIn);
  finally
    bg.Free;
  end;
end;

{ TBabelGoogleRest }

function TBabelGoogleRest.DoTranslate(strIn: string): string;
var
  strUrl, strResult: string;
  nPosA, nPosB: Integer;
begin
  Http1.Request.Referer := 'http://www.marcocantu.com';
  strUrl := Format (
    'http://ajax.googleapis.com/ajax/services/language/translate?' +
    'v=1.0&q=%s&langpair=%s',
    [TIdUri.ParamsEncode (strIn),
    FFromLang + '%7C' + FToLang]); // format hates the %7 !!!
  strResult := Http1.Get(strUrl);
  nPosA := Pos ('"translatedText":', strResult); // begin of JSON data
  if nPosA = 0 then
  begin
    nPosA := Pos ('"responseDetails":', strResult); // begin of JSON data
    nPosA := nPosA + Length ('"responseDetails":');
  end
  else
    nPosA := nPosA + Length ('"translatedText":');

  // shared code
  nPosA := PosEx ('"', strResult, nPosA) + 1; // opening quotes
  nPosB := PosEx ('"', strResult, nPosA) - 1; // end quotes
  Result := Copy (strResult, nPosA, nPosB - nPosA + 1);
end;

end.
